#include once "srb.bi"
#include once "aspi.bi"
#include once "cdrom.bi"
#include once "windows.bi"

declare function CDROM_ScanDevicesCB _
	( _
		byval dinfo as ASPIDEVICEINFO ptr _
	) as integer

declare function CDROM_ReadTOC _
	( _
		byval Adapter as integer, _
		byval Device as integer, _
		byval Lun as integer _
	) as integer

type CDROMDEVICE
	adapter as integer
	device as integer
	lun as integer
end type

type TOC_TRACK field = 1
    reserved1 as ubyte
    adr as ubyte
    track as ubyte
    reserved2 as ubyte
    addr(0 to 3) as ubyte
end type

type TOC_CDROM field = 1
    toclen(0 to 1) as ubyte
    firsttrack as ubyte
    lasttrack as ubyte
    tracks(0 to 99) as toc_track
end type

/'
#define MAXIMUM_NUMBER_TRACKS 100

typedef struct _TRACK_DATA
{
  UCHAR Reserved;
  UCHAR Control : 4;
  UCHAR Adr : 4;
  UCHAR TrackNumber;
  UCHAR Reserved1;
  UCHAR Address[4];
} TRACK_DATA;


typedef struct _CDROM_TOC
{
  UCHAR Length[2];
  UCHAR FirstTrack;
  UCHAR LastTrack;
  TRACK_DATA TrackData[MAXIMUM_NUMBER_TRACKS];
} CDROM_TOC;
'/

dim shared ncdromdevices as integer = 0
dim shared cdromdevices(0 to 15) as CDROMDEVICE
dim shared cdtoc as TOC_CDROM

'' --------------------------------------------------------

''
private function CDROM_ScanDevicesCB _
	( _
		byval dinfo as ASPIDEVICEINFO ptr _
	) as integer


	'' Make this an option for QUIET
	print "Found CDROM (adapter=" & dinfo->adapter _
		& ", device=" & dinfo->device _
		& ", unit=" & dinfo->lun & ")"

	cdromdevices(ncdromdevices).adapter = dinfo->adapter
	cdromdevices(ncdromdevices).device = dinfo->device
	cdromdevices(ncdromdevices).lun = dinfo->lun
	ncdromdevices += 1

	function = 0

end function

''
function CDROM_ScanDevices() as integer
	dim dev as ASPIDEVICEINFO
	ncdromdevices = 0
	function = ASPI_ScanBus( DTYPE_CDROM, @dev, procptr(CDROM_ScanDevicesCB) )
end function

''
private function CDROM_ReadTOC _
	( _
		byval Adapter as integer, _
		byval Device as integer, _
		byval Lun as integer _
	) as integer

	dim t as TOC_CDROM
	dim ExecIO as SRB_ExecSCSICmd
	dim ret as integer
	dim hEvent as HANDLE

	print "Reading TOC... (" & Adapter & "/" & Device & "/" & Lun & ")"
	
	hEvent = CreateEvent( NULL, TRUE, FALSE, NULL )

	ExecIO.SRB_Cmd = SC_EXEC_SCSI_CMD

	ExecIO.SRB_HaID = Adapter
	ExecIO.SRB_Target = Device
	ExecIO.SRB_Lun = Lun

	ExecIO.SRB_Flags = SRB_DIR_IN or SRB_EVENT_NOTIFY

	ExecIO.SRB_BufLen = sizeof(TOC_CDROM)
	ExecIO.SRB_BufPointer = cast( BYTE ptr, @t )

	ExecIO.SRB_SenseLen = SENSE_LEN
	ExecIO.SRB_CDBLen = 10
	ExecIO.CDBByte(0) = SCSI_READ_TOC
	ExecIO.CDBByte(1) = &H2     'MSF mode
	ExecIO.CDBByte(7) = sizeof(TOC_CDROM) \ 256
	ExecIO.CDBByte(8) = sizeof(TOC_CDROM) mod 256

	ExecIO.SRB_PostProc = hEvent

	ResetEvent hEvent
	ret = SendASPI32Command(@ExecIO)

	if (ret = SS_PENDING) then
		WaitForSingleObject hEvent, 20000
	else
		print ASPI_GetErrorMsg( ExecIO.SRB_Status )
	End If

	CloseHandle hEvent

	if( ExecIO.SRB_Status = SS_COMP ) Then

''		print "toc len = " & t.toclen(0) & ", " & t.toclen(1)
''		print "first track = " & t.firsttrack
''		print "last track = " & t.lasttrack

		cdtoc = t

		function = TRUE

	else
		print ASPI_GetErrorMsg( ExecIO.SRB_Status )

	end if

end function

''
function CDROM_ReadTOCByIndex _
	( _
		byval index as integer _
	) as integer

	if( (index >= 0) and (index < ncdromdevices)) then
		function = CDROM_ReadTOC _
			( _
				cdromdevices(index).adapter, _
				cdromdevices(index).device, _
				cdromdevices(index).lun _
			)
	else
		function = FALSE
	end if

end function

''
function CDROM_GetNumTracks() as integer
	if( cdtoc.firsttrack <> 0 and cdtoc.lasttrack <> 0 ) then
		function = cdtoc.lasttrack - cdtoc.firsttrack + 1
	else
		function = 0
	end if
end function

''
function CDROM_ListTOC() as integer

	if( cdtoc.firsttrack <> 0 and cdtoc.lasttrack <> 0 ) then
		dim n as integer
		n = cdtoc.lasttrack - cdtoc.firsttrack + 1
		function = n
		for i as integer = 1 to n + 1
			with cdtoc.tracks(i - cdtoc.firsttrack)
				print .adr, hex(.track), 
				print right( "00" & .addr(1), 2);
				print ":";
				print right( "00" & .addr(2), 2);
				print ":";
				print right( "00" & .addr(3), 2)
			end with
		next
	else
		function = 0
	end if

end function

''
function CDROM_GetTrackOffset( byval i as integer ) as integer

	function = ( cint(cdtoc.tracks(i).addr(1)) * 60 _
				+ cint(cdtoc.tracks(i).addr(2))) * 75 _
				+ cint(cdtoc.tracks(i).addr(3))

end function

''
function CDROM_GetTotalLengthSeconds() as integer

   function = CDROM_GetTrackOffset( CDROM_GetNumTracks() ) \ 75
   
end function



'' --------------------------------------------------------

type TOCOFFSET
	min as integer
	sec as integer
	fra as integer
end type

dim shared tracks(0 to 99) as TOCOFFSET

function cddb_sum( byval n as integer ) as integer

	dim ret as integer = 0

	while (n > 0)
		ret = ret + (n mod 10)
		n = n \ 10
	wend

	function = ret

end function

''
function CDROM_get_cddb_discid() as ulongint

	dim as integer i = 0, t = 0, n = 0, tot_trks = 0
	
	tot_trks = CDROM_GetNumTracks()

	for i as integer = 0 to tot_trks
		tracks(i).min = cdtoc.tracks(i).addr(1)
		tracks(i).sec = cdtoc.tracks(i).addr(2)
		tracks(i).fra = cdtoc.tracks(i).addr(3)
	next

	while (i < tot_trks)
		n = n + cddb_sum((tracks(i).min * 60) + tracks(i).sec)
		i += 1
	wend

	t = ((tracks(tot_trks).min * 60) + tracks(tot_trks).sec) _
	    - ((tracks(0).min * 60) + tracks(0).sec)

	function = ((n mod &hff) shl 24) or (t shl 8) or (tot_trks)

end function

''
function CDROM_get_cddb_query_string() as string

	dim x as string
	dim n as integer

	n = CDROM_GetNumTracks()

	x = "cddb query"
	x &= " " & lcase(hex(CDROM_get_cddb_discid(),8))
	x &= " " & n
	for i as integer = 0 to n-1
		x &= " " & CDROM_GetTrackOffset(i)
	next
	x &= " " & CDROM_GetTotalLengthSeconds()
	
	function = x

end function
