type DWORD as uinteger
type WORD as ushort
type BOOL as integer
type LPSRB as any ptr
type PASPI32BUFF as any ptr
type PDWORD as DWORD ptr
type VOID as ANY

#define SC_HA_INQUIRY               &h00        '' Host adapter inquiry
#define SC_GET_DEV_TYPE             &h01        '' Get device type
#define SC_EXEC_SCSI_CMD            &h02        '' Execute SCSI command

#define SENSE_LEN                   14          '' Default sense buffer length

#define SRB_DIR_IN                  &h08        '' Transfer from SCSI target to host
#define SRB_EVENT_NOTIFY            &h40        '' Enable ASPI event notification

#define SS_PENDING                  &h00        '' SRB being processed
#define SS_COMP                     &h01        '' SRB completed without error
#define SS_ABORTED                  &h02        '' SRB aborted
#define SS_ABORT_FAIL               &h03        '' Unable to abort SRB
#define SS_ERR                      &h04        '' SRB completed with error

#define SS_ABORTED_QUEUE_FROZEN     &h42        '' SRB aborted, target queue frozen
#define SS_ERR_QUEUE_FROZEN         &h44        '' SRB completed with error, target queue frozen
#define SS_BUFFTOBIG_QUEUE_FROZEN   &h4F        '' SRB buffer too big, target queue frozen

#define SS_INVALID_CMD              &h80        '' Invalid ASPI command
#define SS_INVALID_HA               &h81        '' Invalid host adapter number
#define SS_NO_DEVICE                &h82        '' SCSI device not installed

#define SS_ILLEGAL_MODE             &hE2        '' Unsupported Windows mode
#define SS_NO_ASPI                  &hE3        '' No ASPI managers resident
#define SS_FAILED_INIT              &hE4        '' ASPI for windows failed init
#define SS_ASPI_IS_BUSY             &hE5        '' No resources available to execute cmd
#define SS_BUFFER_TO_BIG            &hE6        '' Buffer size to big to handle!
#define SS_BUFFER_TOO_BIG           &hE6        '' Buffer size toobig to handle!
#define SS_MISMATCHED_COMPONENTS    &hE7        '' The DLLs/EXEs of ASPI don't version check
#define SS_NO_ADAPTERS              &hE8        '' No host adapters to manage
#define SS_INSUFFICIENT_RESOURCES   &hE9        '' Couldn't allocate resources needed to init

type SRB_HAInquiry field = 1
    as BYTE        SRB_Cmd                         '' 00/000 ASPI command code = SC_HA_INQUIRY
    as BYTE        SRB_Status                      '' 01/001 ASPI command status byte
    as BYTE        SRB_HaId                        '' 02/002 ASPI host adapter number
    as BYTE        SRB_Flags                       '' 03/003 ASPI request flags
    as DWORD       SRB_Hdr_Rsvd                    '' 04/004 Reserved, MUST = 0
    as BYTE        HA_Count                        '' 08/008 Number of host adapters present
    as BYTE        HA_SCSI_ID                      '' 09/009 SCSI ID of host adapter
    as BYTE        HA_ManagerId(0 to 15)           '' 0A/010 String describing the manager
    as BYTE        HA_Identifier(0 to 15)          '' 1A/026 String describing the host adapter
    as WORD        HA_BufAlignMask                 '' 2A/042 HA_Unique[1-0]
    as BYTE        HA_Flags                        '' 2C/044 HA_Unique[2]
    as BYTE        HA_MaxTargets                   '' 2D/045 HA_Unique[3]
    as DWORD       HA_MaxTransferLength            '' 2E/046 HA_Unique[7-4]
    as DWORD       HA_MaxSGElements                '' 32/050 HA_Unique[11-8]
    as BYTE        HA_Rsvd2(0 to 3)                '' 36/054 HA_Unique[15-12]
''  as BYTE        HA_Unique(0 to 16)              '' 2A/042 Host Adapter Unique parameters++*
    as WORD        HA_Rsvd1                        '' 3A/058 Reserved, MUST = 0
end type

type SRB_GetDevType field = 1
    as BYTE        SRB_Cmd                         '' 00/000 ASPI command code = SC_GET_DEV_TYPE
    as BYTE        SRB_Status                      '' 01/001 ASPI command status byte
    as BYTE        SRB_HaId                        '' 02/002 ASPI host adapter number
    as BYTE        SRB_Flags                       '' 03/003 Reserved, MUST = 0
    as DWORD       SRB_Hdr_Rsvd                    '' 04/004 Reserved, MUST = 0
    as BYTE        SRB_Target                      '' 08/008 Target's SCSI ID
    as BYTE        SRB_Lun                         '' 09/009 Target's LUN number
    as BYTE        SRB_DeviceType                  '' 0A/010 Target's peripheral device type
    as BYTE        SRB_Rsvd1                       '' 0B/011 Reserved, MUST = 0
end type

type SRB_ExecSCSICmd field = 1
  as BYTE  SRB_Cmd                  '' ASPI command code = SC_EXEC_SCSI_CMD
  as BYTE  SRB_Status               '' ASPI command status byte
  as BYTE  SRB_HaId                 '' ASPI host adapter number
  as BYTE  SRB_Flags                '' ASPI request flags
  as DWORD SRB_Hdr_Rsvd             '' Reserved
  as BYTE  SRB_Target               '' Target's SCSI ID
  as BYTE  SRB_Lun                  '' Target's LUN number
  as WORD  SRB_Rsvd1                '' Reserved for Alignment
  as DWORD SRB_BufLen               '' Data Allocation Length
  as BYTE  ptr SRB_BufPointer       '' Data Buffer Pointer
  as BYTE  SRB_SenseLen             '' Sense Allocation Length
  as BYTE  SRB_CDBLen               '' CDB Length
  as BYTE  SRB_HaStat               '' Host Adapter Status
  as BYTE  SRB_TargStat             '' Target Status
  as VOID  ptr SRB_PostProc         '' Post routine
  as BYTE  SRB_Rsvd2(0 to 19)       '' Reserved, MUST = 0
  as BYTE  CDBByte(0 to 15)         '' SCSI CDB
  as BYTE  SenseArea(0 to SENSE_LEN+2-1)  '' Request Sense buffer
end type



'' SCSIDEFS

#define DTYPE_DASD      &h00    '' Disk Device
#define DTYPE_SEQD      &h01    '' Tape Device
#define DTYPE_PRNT      &h02    '' Printer
#define DTYPE_PROC      &h03    '' Processor
#define DTYPE_WORM      &h04    '' Write-once read-multiple
#define DTYPE_CROM      &h05    '' CD-ROM device
#define DTYPE_CDROM     &h05    '' CD-ROM device
#define DTYPE_SCAN      &h06    '' Scanner device
#define DTYPE_OPTI      &h07    '' Optical memory device
#define DTYPE_JUKE      &h08    '' Medium Changer device
#define DTYPE_COMM      &h09    '' Communications device
#define DTYPE_RESL      &h0A    '' Reserved (low)
#define DTYPE_RESH      &h1E    '' Reserved (high)
#define DTYPE_UNKNOWN   &h1F    '' Unknown or no device type

#define SCSI_READ_TOC   &h43    '' Read TOC (O)